import functions from 'firebase-functions/v1';
import api from '@cloudcommerce/api';
import config, { createExecContext } from '../config.js';
import { GET_PUBSUB_TOPIC } from '../const.js';

const { logger } = functions;
const createPubSubFunction = (pubSubTopic, fn, {
  eventMaxAgeMs = 60000, memory = '256MB',
  /* PubSub ordering is ignored on deploys with Firebase CLI,
there is no hard guarantee but max instances 1 is a workaround:
https://stackoverflow.com/questions/70780310/firebase-pubsub-trigger-with-message-ordering */
  maxInstances = 1,
} = {}) => {
  const { httpsFunctionOptions: { region } } = config.get();
  return functions.region(region)
    .runWith({
      failurePolicy: eventMaxAgeMs > 0,
      memory,
      maxInstances,
    })
    .pubsub.topic(pubSubTopic).onPublish((message, context) => {
      const eventAgeMs = Date.now() - Date.parse(context.timestamp);
      if (eventAgeMs > eventMaxAgeMs) {
        logger.warn(`Dropping event ${context.eventId} with age[ms]: ${eventAgeMs}`, {
          data: message.json,
        });
        return Promise.resolve(null);
      }
      return createExecContext(() => fn(message.json, context, message));
    });
};
const createAppEventsFunction = (appNameOrId, fn, options, isSkipMiddleware = false) => {
  let appId;
  if (typeof appNameOrId === 'string') {
    appId = config.get().apps[appNameOrId].appId;
  } else {
    appId = appNameOrId;
  }
  const midd = async (payload, context, message) => {
    const { evName, apiEvent: { resource, resource_id: resourceId, timestamp } } = payload;
    logger.info(`ev/${evName} ${resourceId} at ${timestamp}`, {
      modifiedFields: payload.apiEvent.modified_fields,
      datasetAt: payload.at,
    });
    if (resource && payload.at && Date.now() - payload.at > 600) {
      const { data: apiDoc } = await api.get(`${resource}/${resourceId}`);
      if (!(payload.apiDoc.updated_at > apiDoc.updated_at)) {
        payload.apiDoc = apiDoc;
      }
    }
    return fn(payload, context, message);
  };
  const _fn = isSkipMiddleware === true ? fn : midd;
  return createPubSubFunction(GET_PUBSUB_TOPIC(appId), _fn, options);
};

export default createPubSubFunction;

export { createPubSubFunction, createAppEventsFunction };
// # sourceMappingURL=pubsub.js.map
